
class Builder
  def initialize(store, log)
    @store = store
    @log = log
  end

  def build(target)
    @log.info("checking if #{target} needs a build")
    if not requires_build?(target) then
      @log.info("revision #{target} needs no build")
      return @store.last_item_for_target(target)
    end
    @log.info("it needs one...")
    return new_build_for_target(target)
  end

  def requires_build?(target)          
    return !@store.has_item_for_target?(target) || target.deps_have_changed?
  end
  
  def new_build_for_target(target)
    target.obtain_item(@store)
    build_target(target)
    target.save(@store)
    return target.item
  end

  def build_target(target)
    target.set_progress(true)
    if target.deps_are_successful? then
      target.fire
    else
      @log.warn("there were failed dependencies; no build performed.")
    end
    target.set_progress(false)
  end
end

