
require 'open4'

class ShellFailure < Exception
  attr_reader :exitstatus, :output, :errors

  def initialize(command, exitstatus, output, errors)
    @command = command
    @exitstatus = exitstatus
    @output = output
    @errors = errors
  end

  def to_s
    s = "Command #{@command} failed with exitcode #{@exitstatus}:\n"
    s += "stdout: \n#{@output}\n"
    s += "stderr: \n#{@errors}\n"
    return s
  end

end

module MyShell
  def execute(command)
    cid, _, stdout, stderr = Open4::popen4(command)
    out = stdout.read
    err = stderr.read
    pid, status = Process::waitpid2(cid)
    
    if status != 0 then
      raise ShellFailure.new(command, status, out, err)
    end
    return out, err
  end

  def read(command)
    output, _ = execute(command)
    return output
  end
end
 
