
require 'optparse'
require 'ostruct'
require 'yaml'

class SisyphusClientCommandlineParser
  def self.parse(args)
    options = OpenStruct.new
    options.profile_name = nil
    options.config_url = nil
    options.interval = 3600
    options.roots = []
    options.verbose = false
    options.user = nil
    options.password = nil
    options.dbconf = nil
    options.options_filename = nil
    
    opts = OptionParser.new do |opts|
      opts.banner = "Usage: si-cli [options]"
      opts.separator ""
      opts.separator "Options:"

      opts.on("-p Profile", "--profile Profile",
              "Name of profile used to configure build the process") do |profile_name|
        options.profile_name = profile_name
      end
      
      opts.on("-c Config", "--config Config",
              "Subversion URL pointing to Sisyphus config files") do |config|
        options.config_url = config
      end
      
      opts.on("-r Root", "--root Root",
              "Root package to start build with (may be specified multiple times).") do |root|
        options.roots << root
      end

      opts.on("-o Options", "--options Options", "YAML file containing options.") do |o|
        options.options_filename = o
      end

      opts.on("-i Interval", "--interval Interval",
              "Interval (in s) waited after build session (3600)") do |interval|
        options.interval = interval.to_i
      end

      opts.on("-d Dbconf", "--dbconf Dbconf", "Database config file.") do |d|
        File.open(d) do |f|
          options.dbconf = YAML.load(f)
        end
      end

      opts.on("-U", "--User User", "Subversion user") do |u|
	options.user = u
      end

      opts.on("-P", "--Password Password", "Subversion password") do |p|
	options.password = p
      end

      opts.on("-v", "--[no-]verbose", "Run verbosely") do |v|
	options.verbose = v
      end



      opts.parse!(args)
      
      if not options.options_filename.nil? then
        File.open(options.options_filename) do |f|
          config = YAML.load(f)
          options.profile_name = config['options']['profile']
          options.interval = config['options']['interval']
          options.roots = config['options']['roots']
          options.verbose = config['options']['verbose']

          options.config_url = config['subversion']['url']
          options.user = config['subversion']['username']
          options.password = config['subversion']['password']
          options.dbconf = config['database']
        end
      end

      if options.profile_name.nil? or 
          options.config_url.nil? or 
          options.roots == [] or
          options.dbconf.nil? then
        puts opts
        exit(1)
      end
      
      return options
    end
  end
end
